from __future__ import print_function

import datetime
import logging
import sys
import os
import leip
import hashlib

from mad2.util import get_all_mad_files

lg = logging.getLogger(__name__)

def get_qdhash(filename):
    """
    Provde a quick & dirty hash - a good indication that a file MIGHT have
    changed - but by no means secure.

    It is quick, though.
    """
    sha1sum = hashlib.sha1()
    filesize = os.stat(filename).st_size
    if filesize < 20000:
        with open(filename, 'rb') as F:
            sha1sum.update(F.read())
    else:
        with open(filename, 'rb') as F:
            for x in range(9):
                F.seek(int(filesize * (x/10.0)))
                sha1sum.update(F.read(2000))

            F.seek(-2000, 2)
            sha1sum.update(F.read())

    return sha1sum.hexdigest()


def get_mtime(fn):
    return datetime.datetime.utcfromtimestamp(
        os.stat(fn).st_mtime).isoformat()

def may_have_changed(madfile):
    may_have_changed = False
    if madfile.mad.hash.qdhash:
        qmt = madfile.mad.hash.mtime
        mtime =get_mtime(madfile.fullpath)
        if qmt != mtime:
            may_have_changed = True
    elif madfile.mad.hash.mtime:
        qdh = madfile.mad.hash.qdhash
        cs = get_qdhash(madfile.fullpath)
        if qdh != cs:
            may_have_changed = True
    return may_have_changed

@leip.hook("madfile_post_load", 250)
def hashhelper(app, madfile):
    """
    Calculate a quick&dirty checksum

    """

    if madfile.orphan:
        #cannot deal with orphaned files
        return

    changed = may_have_changed(madfile)
    if changed and not 'sha1' in sys.argv:
        print("{} may have changed! (rerun mad sha1)".format(madfile.fullpath),
              file=sys.stderr)

def hashit(hasher, filename):
    """
    Provde a quick & dirty hash

    this is by no means secure, but quick for very large files, and as long
    as one does not try to create duplicate hashes, the chance is still very
    slim that a duplicate will arise
    """
    h = hasher()
    blocksize = 2**20
    with open(filename, 'rb') as F:
        for chunk in iter(lambda: F.read(blocksize), b''):
            h.update(chunk)
    return h.hexdigest()


@leip.arg('-e', '--echo', action='store_true', help='echo name')
@leip.arg('-f', '--force', action='store_true', help='apply force')
@leip.arg('-w', '--warn', action='store_true', help='warn when skipping')

@leip.arg('file', nargs='*')
@leip.command
def sha1(app, args):
    """
    Calculate a sha1 checksum
    """

    for madfile in get_all_mad_files(app, args):

        if madfile.all.orphan:
            return

        changed = may_have_changed(madfile)

        if not args.force and 'sha1':
            if madfile.hash.sha1:
                if not changed:
                    if args.warn:
                        #exists - and not forcing
                        lg.warning("Skipping sha1 checksum - exists & likely unchanged")
                    continue

        qd = get_qdhash(madfile.filename)
        mtime =get_mtime(madfile.filename)

        madfile.mad.hash.qdhash = qd
        madfile.mad.hash.mtime = mtime

        cs = hashit(hashlib.sha1, madfile.filename)
        madfile.mad.hash.sha1 = cs

        madfile.save()

        if args.echo:
            print(madfile.filename)





from __future__ import print_function

import logging
import os
from pprint import pprint
import re

import leip
import Yaco

from py2neo import neo4j
neo_logger = logging.getLogger(
    "py2neo.packages.httpstream.http")
neo_logger.setLevel(logging.WARNING)

from mad2.util import get_all_mad_files

lg = logging.getLogger(__name__)
#lg.setLevel(logging.INFO)

class GDB:
    #Dummy object that holds data on a database
    def __init__(self, app):
        self.app = app #leip application - access to conf
        if 'uri' in self.app.conf.plugin.neo4j:
            uri = self.app.conf.plugin.neo4j.uri
            self.db = neo4j.GraphDatabaseService(uri)
        else:
            self.db = neo4j.GraphDatabaseService()

        self.file_index = self.db.get_or_create_index(
                neo4j.Node, "File")
        self.sha1_index = self.db.get_or_create_index(
                neo4j.Node, "Sha1")
        self.host_index = self.db.get_or_create_index(
                neo4j.Node, "Host")
        self.user_index = self.db.get_or_create_index(
                neo4j.Node, "User")
        self.project_index = self.db.get_or_create_index(
                neo4j.Node, "Project")

DB = None
def get_db(app):
    global DB
    if DB is None:
        DB = GDB(app)
    return DB

def neo_save_madfile(app, madfile):
    db = get_db(app)
    simple = madfile.simple()
    uri = madfile.all.uri
    user = madfile.all.username
    host = madfile.all.host
    sha1 = madfile.all.hash.sha1
    project = madfile.all.project

    del simple['username']

    if project:
        del simple['project']

    file_node = db.file_index.get_or_create("uri", uri, simple)
    file_node.set_properties(simple)
    file_node.add_labels('file')

    if project:
        proj_node = db.project_index.get_or_create(
            "project", project, {'project' : project} )
        proj_node.add_labels('project')
        proj_node.get_or_create_path("HAS_FILE", file_node)

    if sha1:
        hash_node = db.sha1_index.get_or_create(
            "sha1", sha1, {'sha1' : sha1} )
        hash_node.add_labels('sha1')
        file_node.get_or_create_path("HAS_SHA1", hash_node)

    host_node = db.host_index.get_or_create("host", host,
        {"host" : host})
    host_node.add_labels('host')
    host_node.get_or_create_path("HAS_FILE", file_node)

    user_node = db.host_index.get_or_create("user", user,
        {"user" : user})
    user_node.add_labels('user')
    user_node.get_or_create_path("OWNS_FILE", file_node)


@leip.command
def neo_clean(app, args):
    db = get_db(app)
    db.db.clear()

@leip.hook("madfile_save", 200)
def neo_hook_save(app, madfile):
    lg.debug("start neo4j save")
    neo_save_madfile(app, madfile)

@leip.arg('file', nargs='*')
@leip.command
def neo_save(app, args):
    """
    print a single value from a single file
    """
    for madfile in get_all_mad_files(app, args):
        neo_save_madfile(app, madfile)



from __future__ import print_function

import logging
import sys

import leip
from mad2.util import get_all_mad_files

lg = logging.getLogger(__name__)


#@leip.arg('file', nargs='*')
#@leip.arg('category', help='category to assign to these files')
# @leip.commandName('=')
# def apply_category(app, args):
#     """
#     apply a predefined category to a (set of) file(s)
#     """
#     if not args.category in app.conf.template:
#         print("Invalid category: {0}".format(args.category))
#         print("Choose from:")
#         for cat in app.conf.template:
#             print(" - {0}".format(cat))
#         sys.exit()
#     template_data = app.conf.template[args.category]
#     for madfile in get_all_mad_files(app, args):
#         madfile.mad.update(template_data)
#         madfile.save()



import logging
import os

import jinja2

import Yaco

from mad2.exception import MadPermissionDenied

lg = logging.getLogger(__name__)
#lg.setLevel(logging.DEBUG)


def dummy_hook_method(*args, **kw):
    return None


class MadFile(object):
    """

    """
    def __init__(self,
                 filename,
                 base=Yaco.Yaco(),
                 hook_method=dummy_hook_method):

        self.dirmode = False
        dirname = os.path.dirname(filename)
        basename = os.path.basename(filename)

        lg.debug("Instantiating a madfile for '{}' / '{}'".format(dirname, basename))

        if os.path.isdir(filename):
            self.dirmode = True
            maddir = os.path.join(os.path.abspath(filename), '.mad', 'config')
            if not os.path.exists(maddir):
                os.makedirs(maddir)
            lg.debug("'{}' is a dir".format(filename))
            madname = os.path.join(maddir, '_root.config')
            filename = filename
        else:
            #looking at a filename
            if basename[-4:] == '.mad':

                if basename[0] == '.':
                    basename = basename[1:-4]
                else:
                    #old style - prob needs to go
                    basename = basename[:-4]

                madname = filename
                filename = os.path.join(dirname, basename)
            else:
                filename = filename
                madname = os.path.join(dirname, '.' + basename + '.mad')

        #lg.debug("madname: {}".format(madname))
        #lg.debug("filename: {}".format(filename))

        if os.path.exists(madname) and not os.access(madname, os.R_OK):
            raise MadPermissionDenied()

        #must make a copy - otherwise we're overwriting the same
        #object for the next instantiation
        self.all = base.copy()

        #for local data
        self.mad = Yaco.Yaco()

        self.all.filename = filename
        self.all.dirname = os.path.abspath(dirname)
        self.all.basename = basename
        self.all.madname = madname

        if os.path.exists(madname) and not os.path.exists(filename):
            lg.warning("Orphaned mad file: {}".format(madname))
            self.all.orphan = True

        self.hook_method = hook_method

        self.load()

    def __str__(self):
        return '<mad2.madfile.MadFile {}>'.format(self.all.madname)

    def get(self, key, default):
        return self.all.get(key, default)

    def __getitem__(self, item):
        return self.all[item]

    __getattr__ = __getitem__

    def keys(self):
        return self.all
        sys.exit()

    def simple(self):
        rv = dict(self.all)
        kys = rv.keys()
        for k in kys:
            if not rv[k]:
                del rv[k]
            elif isinstance(rv[k], dict):
                del rv[k]
        return rv
    def data(self, on_top_of={}):
        """Render data into a dict like format
        """
        lg.critical("really - using this??")
        if isinstance(on_top_of, Yaco.Yaco) or \
            isinstance(on_top_of, Yaco.PolyYaco):
            data = on_top_of.simple()

        data.update(self.all.simple())
        data.update(self.mad.simple())
        return data

    def render(self, text, base):

        jenv = jinja2.Environment(
            undefined=jinja2.DebugUndefined )

        rendered = text
        iteration = 0

        while '{{' in rendered or '{%' in rendered:
            #print('1', rendered, y['rabbit'])
            if iteration > 0 and rendered == last:
                #no improvement
                break
            last = rendered
            template = jenv.from_string(rendered)
            rendered = template.render(self.all)
            iteration += 1

        return rendered

    def load(self):
        #print(self.madname)

        self.hook_method('madfile_pre_load', self)
        if os.path.exists(self.madname):
            lg.debug("loading madfile {0}".format(self.madname))
            self.mad.load(self.madname)
            self.all.update(self.mad)

        self.hook_method('madfile_load', self)
        self.hook_method('madfile_post_load', self)

    def save(self):
        self.hook_method('madfile_save', self)
        try:
            lg.debug("saving to %s" % self.madname)
            self.mad.save(self.madname)
        except IOError, e:
            if e.errno == 63:
                lg.warning("Can't save - filename too long: {}"\
                        .format(self.fullpath))
            else:
                raise

        self.hook_method('madfile_post_save', self)

    def pretty(self):
        return self.mad.pretty()



import logging
import os
import re
import select
import sys

from mad2.exception import MadPermissionDenied, MadNotAFile
from mad2.madfile import MadFile

lg = logging.getLogger(__name__)
#lg.setLevel(logging.DEBUG)

##
## Helper function - instantiate a madfile, and provide it with a
## method to run hooks
##
def get_mad_file(app, filename):
    """
    Instantiate a mad file & add hooks
    """
    global CACHE

    lg.debug("instantiating madfile for {0}".format(filename))
    return  MadFile(filename, base=app.conf.madfile, hook_method=app.run_hook)

def to_mad(fn):
    if '/' in fn:
        a, b = fn.rsplit('/')
        return os.path.join(a, '.{}.mad'.format(b))
    else:
        return '.{}.mad'.format(fn)

def get_filenames(args):
    """
    Get all incoming filenames
    """
    filenames = []

    demad = re.compile(r'^(?P<path>.*?/)?\.(?P<fn>[^/].+)\.mad$')
    def demadder(m):
        if not m.group('path') is None:
            return '{}{}'.format(m.group('path'), m.group('fn'))
        else:
            return m.group('fn')

    if 'file'in args and len(args.file) > 0:
        filenames.extend([demad.sub(demadder, x)
                         for x in args.file
                         if (len(x) > 0 and not '.mad/' in x)])
    else:
        #nothing in args - see if there is something on stdin
        filenames.extend(
                [demad.sub(demadder, x)
                 for x in sys.stdin.read().split("\n")
                 if (len(x) > 0 and not '.mad/' in x)])

    filenames = sorted(list(set(filenames)))

    #remove directories as well
    filenames = [x for x in filenames if not '.mad' in x]

    return filenames

def get_all_mad_files(app, args):
    """
    get input files from sys.stdin and args.file
    """
    for filename in get_filenames(args):
        try:
            maf = get_mad_file(app, filename)
            #print(maf.madname)
            yield maf
        except MadNotAFile:
            pass
        except MadPermissionDenied:
            lg.warning("Permission denied: {}".format(
                filename))

def boolify(v):
    """
    return a boolean from a string
    yes, y, true, True, t, 1 -> True
    otherwise -> False
    """
    return v.lower() in ['yes', 'y', 'true', 't', '1']

def render(txt, data):

    env = moa.moajinja.getStrictEnv()
    renconf = self.render()
    templ = env.from_string(value)
    try:
        rv = templ.render(renconf)
        return rv
    except jinja2.exceptions.UndefinedError:
        return value
    except jinja2.exceptions.TemplateSyntaxError:
        return value



from __future__ import print_function

import datetime
import logging
import os

import leip
from mad2.util import  get_all_mad_files

lg = logging.getLogger(__name__)

def execute(app, madfile, cl, dry=False):
    """
    execute a command line in the context of this object

    :param dry: do a dry run
    :type dry: boolean
    """
    lg.info("executing: {0}".format(cl))
    if dry:
        print(cl)
    else:
        os.system(cl)
        t = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        h = madfile.mad.execute.history[t]
        h.cl = cl
        madfile.save()
#        madfile.execute.history:
#           madfile.execute.history = []           madfile.execute.history.extend(executed)
#     madfile.save()


@leip.arg('file', nargs='*')
@leip.arg('comm', metavar='command', help='predefined command to execute')
@leip.arg('-d', '--dry', help='dry run', action='store_true')
@leip.command
def x(app, args):
    """
    Execute a command
    """
    command = args.comm
    lg.info("command to execute: {0}".format(command))

    for madfile in get_all_mad_files(app, args):
        cl = madfile.render(madfile.x[command], app.conf)
        execute(app, madfile, cl, dry=args.dry)



# -*- coding: utf-8 -*-
from __future__ import print_function,  unicode_literals

from collections import Counter
import logging
import os
from signal import signal, SIGPIPE, SIG_DFL
import subprocess
import sys
import tempfile

from dateutil.parser import parse as dateparse

import leip
import Yaco
import mad2.ui

from mad2.util import  get_mad_file, get_all_mad_files

#Ignore SIG_PIPE and don't throw exceptions
#otherwise it crashes when you pipe into, for example, head
#see http://newbebweb.blogspot.be/2012/02/python-head-ioerror-errno-32-broken.html
#see http://docs.python.org/library/signal.html
signal(SIGPIPE, SIG_DFL)

lg = logging.getLogger(__name__)

def dispatch():
    """
    Run the app - this is the actual application entry point
    """
    app.run()


##
## define Mad commands
##


@leip.arg('-f', '--force', action='store_true', help='apply force')
@leip.arg('-p', '--prompt', action='store_true', help='show a prompt')
@leip.arg('-e', '--editor', action='store_true', help='open an editor')
@leip.arg('file', nargs='*')
@leip.arg('value', help='value to set', nargs='?')
@leip.arg('key', help='key to set')
@leip.command
def set(app, args):
    """
    Set a key/value for one or more files.

    Use this command to set a key value pair for one or more files.

    This command can take the following forms::

        mad set project test genome.fasta
        ls *.fasta | mad set project test
        find . -size +10k | mad set project test


    """

    key = args.key
    val = args.value

    if args.prompt or args.editor:
        if not args.value is None:
            # when asking for a prompt - the value is assumed to be a file, and
            # needs to be pushed into args.file
            args.file = [args.value] + args.file

    madfiles = []
    for m in get_all_mad_files(app, args):
        #print ('xxxxxxx', m, m.madname)
        madfiles.append(m)

    if val is None and not (args.prompt or args.editor):
        args.prompt = True

    if args.prompt or args.editor:
        # get a value from the user

        default = ''
        #Show a prompt asking for a value
        if len(madfiles) == 1:
            data = madfiles[0].data(app.conf)
            default = madfiles[0].mad.get(key, "")
        else:
            data = app.conf.simple()

        if args.prompt:
            sys.stdin = open('/dev/tty')
            val = mad2.ui.askUser(key, default, data)
            sys.stdin = sys.__stdin__
        elif args.editor:
            editor = os.environ.get('EDITOR','vim')
            tmp_file = tempfile.NamedTemporaryFile('wb', delete=False)

            #write default value to the tmp file
            if default:
                tmp_file.write(default + "\n")
            else:
                tmp_file.write("\n")
            tmp_file.close()

            tty = open('/dev/tty')

            subprocess.call('{} {}'.format(editor, tmp_file.name),
                stdin=tty, shell=True)
            sys.stdin = sys.__stdin__


            #read value back in
            with open(tmp_file.name, 'r') as F:
                #removing trailing space
                val = F.read().rstrip()
            #remove tmp file
            os.unlink(tmp_file.name)


    lg.debug("processing %d files" % len(madfiles))

    list_mode = False
    if key[0] == '+':
        lg.debug("treating {} as a list".format(key))
        list_mode = True
        key = key[1:]

    keywords = app.conf.keywords
    keyinfo = keywords[key]
    keytype = keyinfo.get('type', 'str')

    if list_mode and keyinfo.cardinality == '1':
        print("Cardinality == 1 - no lists!")
        sys.exit(-1)
    elif keyinfo.cardinality == '+':
        list_mode = True


    if not args.force and not key in keywords:
        print("invalid key: {0} (use -f?)".format(key))
        sys.exit(-1)


    if keytype == 'int':
        try:
            val = int(val)
        except ValueError:
            lg.error("Invalid integer: %s" % val)
            sys.exit(-1)
    elif keytype == 'float':
        try:
            val = float(val)
        except ValueError:
            lg.error("Invalid float: %s" % val)
            sys.exit(-1)
    elif keytype == 'boolean':
        if val.lower() in ['1', 'true', 't', 'yes', 'aye', 'y', 'yep']:
            val = True
        elif val.lower() in ['0', 'false', 'f', 'no', 'n', 'nope']:
            val = False
        else:
            lg.error("Invalid boolean: %s" % val)
            sys.exit(-1)
    elif keytype == 'date':
        try:
            val = dateparse(val)
        except ValueError:
            lg.error("Invalid date: %s" % val)
            sys.exit(-1)
        lg.warning("date interpreted as: %s" % val)

    if keytype == 'restricted' and \
            not val in keyinfo.allowed:
        print("Value '{0}' not allowed".format(val))
        sys.exit(-1)

    for madfile in madfiles:
        lg.debug('madmadmad')
        if list_mode:
            if not key in madfile.mad:
                oldval = []
            else:
                oldval = madfile.mad[key]
                if not isinstance(oldval, list):
                    oldval = [oldval]
            madfile.mad[key] = oldval + [val]
        else:
            #not listmode
            madfile.mad[key] = val

        madfile.save()

##
## define show
##
@leip.arg('-y', '--yaml', action='store_true')
@leip.arg('file', nargs='*')
@leip.command
def show(app, args):
    i = 0

    for madfile in get_all_mad_files(app, args):
        if i > 0:
            print('---')

        if args.yaml:
            print(madfile.all.pretty())
        else:
            for k in sorted(madfile.all.keys()):
                v = madfile.all[k]

                if isinstance(v, Yaco.Yaco):
                    continue
                print("{}\t{}".format(k,v))

        i += 1

##
## define unset
##
@leip.arg('file', nargs="+")
@leip.arg('key')
@leip.command
def unset(app, args):
    lg.debug("unsetting: %s".format(args.key))
    key = args.key
    keywords = app.conf.keywords
    keyinfo = keywords[key]
    if keyinfo.cardinality =='+':
        print("Not implemented - unsetting keys with cardinality > 1")
        sys.exit(-1)

    for madfile in get_all_mad_files(app, args):
        #print(madfile)
        #print(madfile.mad.pretty())
        if args.key in madfile.mad:
            del(madfile.mad[args.key])
        madfile.save()


@leip.commandName('key')
def madkeylist(app, args):
    kyw = sorted(app.conf.keywords)
    mxlen = str(max([len(x) for x in kyw])+1)
    for c in sorted(app.conf.keywords):
        print(("{:<" + mxlen + "} {}").format(c, app.conf.keywords[c].description))


@leip.commandName('category')
def madcatlist(app, args):
    kyw = sorted(app.conf.keywords.category.allowed)
    mxlen = str(max([len(x) for x in kyw])+1)
    for c in sorted(kyw):
        dsc = app.conf.keywords.category.allowed[c]
        print(("{:<" + mxlen + "} {}").format(c, dsc))
        tmpl = app.conf.template[c]
        if not tmpl: continue
        mxlen2 = str(max([len(x) for x in tmpl])+1)
        for k in sorted(tmpl):
            print(("  - {:<" + mxlen2 + "} {}").format(k, app.conf.template[c][k]))


@leip.arg('section', help='section to print', default='', nargs='?')
@leip.command
def sysconf(app, args):
    c = app.conf[args.section]
    print(c.pretty())

@leip.arg('comm', metavar='command', help='command to check')
@leip.command
def has_command(app, args):
    comm = args.comm
    lg.debug("checking if we know command %s" % comm)
    if comm in app.leip_commands:
        sys.exit(0)
    else:
        sys.exit(-1)

##
## Instantiate the app and discover hooks & commands
##

#trail of config files???
config_files = [
    'pkg://mad2/etc/*.config',
    '/etc/mad/',
    '~/.config/mad/']

for c in config_files:
    lg.debug("using config file: {}".format(c))

app = leip.app(name='mad2', set_name=None,
               config_files = config_files)

#discover hooks in this module!
app.discover(globals())



from __future__ import print_function, division

from datetime import datetime
import os
import logging
import socket

import Yaco
import leip
from pwd import getpwuid

lg = logging.getLogger(__name__)

EXTENSION_DATA = None
RECURSE_CACHE = {}

def get_fiex(app):
    global EXTENSION_DATA
    if not EXTENSION_DATA is None:
        return EXTENSION_DATA

    EXTENSION_DATA = {}
    for ft in app.conf.filetype:
        for ext in app.conf.filetype.get(ft).extensions:
            EXTENSION_DATA[ext] = ft

    return EXTENSION_DATA


def apply_file_format(app, madfile, filename=None):

    extension_data = get_fiex(app)

    if filename is None:
        filename = madfile.all.basename

    splitter = filename.rsplit('.', 1)
    if len(splitter) != 2:
        return

    base, ext = splitter

    #this ensures that the innermost extension seen is stored
    madfile.all.extension = ext

    if not ext in extension_data:
        return

    ft = extension_data[ext]
    ftinfo = app.conf.filetype.get(ft)

    lg.debug("identified filetype %s" % ft)
    if ftinfo.template:
        template = app.conf.template.get(ftinfo.template)
        madfile.all.update(template)

    if ftinfo.get('continue', False):
        lg.debug("contiue filetype disocvery on: %s" % base)
        apply_file_format(app, madfile, base)


@leip.hook("madfile_pre_load")
def recursive_dir_data(app, madfile):
    lg.debug("start pre load for {}".format(madfile.all.filename))
    here = madfile.all.dirname.rstrip('/')
    conf = []

    #print ('x' * 80)
    #find existsing configurations
    last = here
    while True:
        try:
            assert(os.path.isdir(here))
        except AssertionError:
            print(last, here)
            print(madfile.all.pretty())
            raise
        here_c = os.path.join(here, '.mad', 'config')
        if os.path.exists(here_c):
            conf.append(here_c)

        parent = os.path.dirname(here)
        if parent == '/': #no config in the root - that would be evil!
            break
        last = here
        here = parent


    #load (or get from cache)
    for c in conf[::-1]:
        fullname = os.path.expanduser(os.path.abspath(c))
        if fullname in RECURSE_CACHE:
            y = RECURSE_CACHE[fullname]
        else:
            y = Yaco.YacoDir(fullname)
        madfile.all.update(y)

@leip.hook("madfile_post_load")
def onthefly(app, madfile):

    if sorted(list(madfile.mad.keys())) == ['hash']:
        madfile.all.annotated = False
    else:
        madfile.all.annotated = True

    lg.debug("running onthelfy")
    madfile.all.fullpath = os.path.abspath(madfile.filename)
    madfile.all.fullmadpath = os.path.abspath(madfile.madname)

    lg.debug("get fqdn")
    madfile.all.host = socket.gethostname()

    madfile.all.uri = "file://{}{}".format(
        madfile.all.host, madfile.all.fullpath)
    if madfile.orphan:
        #if file is orphaned - little we can do
        return

    filestat = os.stat(madfile.all.fullpath)
    madfile.all.filesize = filestat.st_size
    try:
        userinfo = getpwuid(filestat.st_uid)
    except KeyError:
        #cannot find username based on uid
        madfile.all.userid = str(filestat.st_uid)
        madfile.all.username = str(filestat.st_uid)
    else:
        madfile.all.userid = userinfo.pw_name
        madfile.all.username = userinfo.pw_gecos

    mtime = datetime.utcfromtimestamp(
        filestat.st_mtime)
    atime = datetime.utcfromtimestamp(
        filestat.st_atime)

    madfile.all.atime = atime.isoformat()
    madfile.all.atime_simple = atime.strftime("%Y/%m/1")

    madfile.all.mtime = mtime.isoformat()
    madfile.all.mtime_simple = mtime.strftime("%Y/%m/1")

    apply_file_format(app, madfile)

    lg.debug("finished onthefly")



from __future__ import print_function

import argparse
import logging
import subprocess as sp
import shutil

import leip

from mad2.util import to_mad

lg = logging.getLogger(__name__)

@leip.arg('args', nargs=argparse.REMAINDER)
@leip.command
def cp(app, args):
    """
    Fake copy - and make sure that the .mad files go along.
    """
    a = list(args.args)
    if not '-v' in a:
        a = ['-v'] + a
    a = ['cp'] + a
    P = sp.Popen(a, stdout=sp.PIPE)
    out, _ = P.communicate()
    froms, tos = [], []
    for line in out.split("\n"):
        line = line.strip()
        if not line: continue
        if line.count(' -> ') != 1:
            lg.warning("invalid cp output: ({}) {}".format(
                line.count(' -> '), line))
            continue
        fr, to = line.split(' -> ')
        froms.append(fr)
        tos.append(to)

    for fr, to in zip(froms, tos):
        frbase = fr.rsplit('/')[-1]
        if frbase[0] == '.' and frbase[-4:] == '.mad':
            #if this is a mad file, ignore
            continue

        frmad = to_mad(fr)
        tomad = to_mad(to)
        if frmad in froms:
            #if the mad file is already copied - don't bother
            continue
        shutil.copy(frmad, tomad)
        lg.debug("Copying mad file {} to {}".format(frmad, tomad))


